<?php

namespace Drupal\openai_client;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use OpenAI;
use OpenAI\Client;

/**
 * Service description.
 */
class OpenAIClientWrapper {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The uuid service.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuid;

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs an OpenAIClientWrapper object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Component\Uuid\UuidInterface $uuid
   *   The uuid service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   */
  public function __construct(ConfigFactoryInterface $config_factory, UuidInterface $uuid, MessengerInterface $messenger) {
    $this->configFactory = $config_factory;
    $this->uuid = $uuid;
    $this->messenger = $messenger;
  }

  /**
   * Get a new client with token set.
   *
   * @param string $token Optional API token, by default use the module set token.
   *
   * @return \OpenAI\Client
   */
  public function getClient(string $token = ''): Client {
    if (empty($token)) {
      $token = $this->getApiToken();
    }
    return OpenAI::client($token);
  }

  /**
   * Get available AI models.
   *
   * @return array
   */
  public function getModels(): array {
    $response = $this->getClient()->models()->list();
    return $response->toArray()['data'];
  }

  /**
   * Has module API token set?
   *
   * @return bool
   */
  public function hasToken():bool {
    return !empty($this->getApiToken());
  }

  /**
   * Do a direct query to default chat model and return the response text.
   *
   * @param string $prompt
   *
   * @return string
   */
  public function fastChatQuery(string $prompt): string {
    try {
      $message = [
        'model' => $this->getDefaultModel(),
        'messages' => [['role' => 'user', 'content' => $prompt]],
      ];

      $response = $this->getClient()->chat()->create($message);

      return $response->choices[0]->message->content;
    } catch (\Exception $e) {
      $this->messenger->addError($e->getMessage());
      return $prompt;
    }
  }

  /**
   * Get default module chat model.
   *
   * @return array|mixed|null
   */
  public function getDefaultModel() {
    $config = $this->configFactory->get('openai_client.settings');
    return $config->get('api-chat-model');
  }

  /**
   * Get OpenAI API token.
   *
   * @return array|mixed|null
   */
  protected function getApiToken() {
    $config = $this->configFactory->get('openai_client.settings');
    return $config->get('api-token');
  }
}
